pub enum MetaAddrChange {
NewInitial {
addr: PeerSocketAddr,
},
NewGossiped {
addr: PeerSocketAddr,
untrusted_services: PeerServices,
untrusted_last_seen: DateTime32,
},
NewLocal {
addr: PeerSocketAddr,
},
UpdateAttempt {
addr: PeerSocketAddr,
},
UpdateConnected {
addr: PeerSocketAddr,
services: PeerServices,
is_inbound: bool,
},
UpdatePingSent {
addr: PeerSocketAddr,
ping_sent_at: Instant,
},
UpdateResponded {
addr: PeerSocketAddr,
rtt: Option<Duration>,
},
UpdateFailed {
addr: PeerSocketAddr,
services: Option<PeerServices>,
},
UpdateMisbehavior {
addr: PeerSocketAddr,
score_increment: u32,
},
}Expand description
A change to an existing MetaAddr.
Variants
NewInitial
Creates a MetaAddr for an initial peer.
Fields
addr: PeerSocketAddrNewGossiped
Creates a new gossiped MetaAddr.
NewLocal
Creates new local listener MetaAddr.
Fields
addr: PeerSocketAddrUpdateAttempt
Updates an existing MetaAddr when an outbound connection attempt
starts.
Fields
addr: PeerSocketAddrUpdateConnected
Updates an existing MetaAddr when we’ve made a successful connection with a peer.
UpdatePingSent
Updates an existing MetaAddr when we send a ping to a peer.
UpdateResponded
Updates an existing MetaAddr when a peer responds with a message.
UpdateFailed
Updates an existing MetaAddr when a peer fails.
UpdateMisbehavior
Updates an existing MetaAddr when a peer misbehaves such as by advertising
semantically invalid blocks or transactions.
Implementations
Sourceimpl MetaAddrChange
impl MetaAddrChange
Sourcepub fn addr_strategy(addr: PeerSocketAddr) -> BoxedStrategy<Self>
pub fn addr_strategy(addr: PeerSocketAddr) -> BoxedStrategy<Self>
Returns a strategy which generates changes for addr.
addr is typically generated by the canonical_peer_addr strategy.
Sourcepub fn addr_changes_strategy(
max_addr_change: usize,
) -> BoxedStrategy<(MetaAddr, Vec<MetaAddrChange>)>
pub fn addr_changes_strategy( max_addr_change: usize, ) -> BoxedStrategy<(MetaAddr, Vec<MetaAddrChange>)>
Returns a strategy which generates a MetaAddr, and a vector of up to
max_addr_change changes.
The address and the changes all have matching PeerSocketAddrs.
Sourcepub fn ready_outbound_strategy_ip() -> BoxedStrategy<IpAddr>
pub fn ready_outbound_strategy_ip() -> BoxedStrategy<IpAddr>
Create a strategy that generates IpAddrs for MetaAddrChanges which are ready for
outbound connections.
Sourcepub fn ready_outbound_strategy_port() -> BoxedStrategy<u16>
pub fn ready_outbound_strategy_port() -> BoxedStrategy<u16>
Create a strategy that generates port numbers for MetaAddrs which are ready for
outbound connections.
Currently, all generated MetaAddrs are the NeverAttemptedGossiped variant.
TODO: Generate all MetaAddr variants, and give them ready fields.
Sourceimpl MetaAddrChange
impl MetaAddrChange
Sourcepub fn addr(&self) -> PeerSocketAddr
pub fn addr(&self) -> PeerSocketAddr
Return the address for this change.
Sourcepub fn set_addr(&mut self, new_addr: PeerSocketAddr)
pub fn set_addr(&mut self, new_addr: PeerSocketAddr)
Set the address for this change to new_addr.
This method should only be used in tests.
Sourcepub fn untrusted_services(&self) -> Option<PeerServices>
pub fn untrusted_services(&self) -> Option<PeerServices>
Return the untrusted services for this change, if available.
Sourcepub fn untrusted_last_seen(&self, now: DateTime32) -> Option<DateTime32>
pub fn untrusted_last_seen(&self, now: DateTime32) -> Option<DateTime32>
Return the untrusted last seen time for this change, if available.
Sourcepub fn last_attempt(&self, now: Instant) -> Option<Instant>
pub fn last_attempt(&self, now: Instant) -> Option<Instant>
Return the last attempt for this change, if available.
Sourcepub fn last_response(&self, now: DateTime32) -> Option<DateTime32>
pub fn last_response(&self, now: DateTime32) -> Option<DateTime32>
Return the last response for this change, if available.
Sourcepub fn ping_sent(&self) -> Option<Instant>
pub fn ping_sent(&self) -> Option<Instant>
Return the timestamp when a ping was last sent, if available.
Sourcepub fn ping_sent_at(&self) -> Option<Instant>
pub fn ping_sent_at(&self) -> Option<Instant>
Returns the timestamp when a ping was last sent, if available.
Sourcepub fn last_failure(&self, now: Instant) -> Option<Instant>
pub fn last_failure(&self, now: Instant) -> Option<Instant>
Return the last failure for this change, if available.
Sourcepub fn peer_addr_state(&self) -> PeerAddrState
pub fn peer_addr_state(&self) -> PeerAddrState
Return the peer connection state for this change.
Sourcepub fn into_new_meta_addr(
self,
instant_now: Instant,
local_now: DateTime32,
) -> MetaAddr
pub fn into_new_meta_addr( self, instant_now: Instant, local_now: DateTime32, ) -> MetaAddr
Returns the corresponding MetaAddr for this change.
Sourcepub fn misbehavior_score(&self) -> u32
pub fn misbehavior_score(&self) -> u32
Returns the misbehavior score increment for the current change.
Sourcepub fn is_inbound(&self) -> bool
pub fn is_inbound(&self) -> bool
Returns whether this change was created for a new inbound connection.
Sourcepub fn local_listener_into_new_meta_addr(
self,
local_now: DateTime32,
) -> MetaAddr
pub fn local_listener_into_new_meta_addr( self, local_now: DateTime32, ) -> MetaAddr
Returns the corresponding MetaAddr for a local listener change.
This method exists so we don’t have to provide an unused Instant to get a local
listener MetaAddr.
Panics
If this change is not a MetaAddrChange::NewLocal.
Sourcepub fn apply_to_meta_addr(
&self,
previous: impl Into<Option<MetaAddr>>,
instant_now: Instant,
chrono_now: DateTime<Utc>,
) -> Option<MetaAddr>
pub fn apply_to_meta_addr( &self, previous: impl Into<Option<MetaAddr>>, instant_now: Instant, chrono_now: DateTime<Utc>, ) -> Option<MetaAddr>
Apply this change to a previous MetaAddr from the address book,
producing a new or updated MetaAddr.
If the change isn’t valid for the previous address, returns None.